-- globals
local tostring_ = tostring
local getn = table.getn
local math_floor = math.floor
local math_abs = math.abs
local math_sin								= math.sin
local math_cos								= math.cos
local math_rad								= math.rad
local GetPlayerFacing_						= GetPlayerFacing
local GetPlayerMapPosition_ = GetPlayerMapPosition
local sqrt = math.sqrt
local math_deg = math.deg

-- Isolate the environment
local folder, core = ...
setfenv(1, core)
-- local

local DEBUG = false
--[===[@debug@
DEBUG = true
--@end-debug@]===]

--~ local whiteText			= "|cffffffff%s|r"
local strDebugFrom		= "|cffffff00[%s]|r" --Yellow function name. help pinpoint where the debug msg is from.
local strWhiteBar		= "|cffffff00 || |r" -- a white bar to seperate the debug info.
-----------------------------
function Debug(from, ...)	--
-- simple print function.	--
------------------------------
	if DEBUG == false then
		return 
	end
	local tbl  = {...}
	local msg = tostring_(tbl[1])
	for i=2,getn(tbl) do 
		msg = msg..strWhiteBar..tostring_(tbl[i])
	end
	echo(strDebugFrom:format(from).." "..msg)
end

local colouredName		= "|cff7f7f7f{|r|cffff0000TR|r|cff7f7f7f}|r "
function echo(...)
	local tbl  = {...}
	local msg = tostring_(tbl[1])
	for i=2,getn(tbl) do 
		msg = msg..strWhiteBar..tostring_(tbl[i])
	end
	
	local cf = _G["ChatFrame1"]
	if cf then
		cf:AddMessage(colouredName..msg,.7,.7,.7)
	end
end

------------------------------------------------------------------
function Round(num, zeros)										--
-- zeroes is the number of decimal places. eg 1=*.*, 3=*.***	--
------------------------------------------------------------------
	return math_floor( num * 10 ^ (zeros or 0) + 0.5 ) / 10 ^ (zeros or 0)
end

----------------------------------------------------------------------------------
function GetMyYardCoords()														--
-- Returns our coords in yards instead of map coords.							--
-- Every map has coords of 0-100, but each zone is a different size in yards.	--
----------------------------------------------------------------------------------
	local mX, mY = GetPlayerMapPosition_("player")
	return CoordsToYardCoords(mX, mY)
end

------------------------------------------
function CoordsToYardCoords(x, y)		--
-- Convert map coords to yard coords.	--
------------------------------------------
	local found, width, height = core:GetZoneSize()
	local yX = width * x
	local yY = height * y
	return yX, yY
end

------------------------------------------
function YardCoordsToMapCoords(sX, sY)	--
-- Convert yard coords to map coords.	--
------------------------------------------
	local found, width, height = core:GetZoneSize()
	local tX = sX / width
	local tY = sY / height
	return tX, tY
end

--[[math.deg(GetPlayerFacing())
	North = 0/360
	West = 90
	South = 180
	East = 270]]
----------------------------------
function MyHeading()			--
-- Returns heading in degrees.	--
----------------------------------
	return math_deg(GetPlayerFacing_())
end
----------------------------------------------------------------------------------
function GetTotemCoords(direction)												--
-- Input a heading and this returns the coords 3 yards away in that direction.	--
----------------------------------------------------------------------------------
	--Get our current coords in yards.
	local x, y = GetMyYardCoords() --Get our coords in yards.

	local heading = MyHeading() + direction --Adjust the heading to compensate for our current heading.
	if (heading < 0) then --Make sure the heading is 0-360. This may not be needed but helped in testing.
		heading = heading + 360;
	elseif heading > 360 then
		heading = heading - 360;
	end
	
	--Get the coords of where the totem spawned, 3 yards away from us.
	local tX, tY = GetDirectionCoords(x, y, heading, spawnDistance)

	--Convert the totem coords back to map coords.
	local cX, cY = YardCoordsToMapCoords(tX, tY)
	return cX, cY
end

----------------------------------------------------------------------------------
function SwitchEW(angle)														--
-- This function switchs east and west directions.								--
----------------------------------------------------------------------------------
	local z = 0 - angle;
	local t = 0 - math_abs(z);
	if t < 0 then
		t = t + 360;
	end
	return t
end


--------------------------------------------------------------------------------------
function GetDirectionCoords(mX, mY, direction, distance)							--
-- Returns a set of coords in a direction and distance of another set of coords.	--
-- Many thanks to Granola of Baelgun server for helping me.							--
--------------------------------------------------------------------------------------
	local direction = direction + 90 -- cos & sin see 360/0deg as east, but GetPlayerFacing sees it at north. We need to adjust for this.
	local direction = SwitchEW(direction) --This seems to work, I don't know why.
	local rad = math_rad(direction)
	local tX = mX + distance * math_cos(rad)
	local tY = mY + distance * math_sin(rad)
	return tX, tY
end

------------------------------------------------------------------------------
function GetMinimapCoordOffset(self, dist, xDist, yDist)					--
--Returns the offset coords where a icon should be placed on the minimap.	--
------------------------------------------------------------------------------
	local map = Minimap
	local mapWidth = map:GetWidth()
	local mapHeight = map:GetHeight()
	
	local diameter = (minimapSize[indoors and "indoor" or "outdoor"][map:GetZoom()] * 2)
	local xScale = diameter / mapWidth;
	local yScale = diameter / mapHeight;
	
	
	local mapRadius = diameter / 2;
	local iconDiameter = ((self:GetWidth() / 2) + 3) * xScale;
	
	local maxDist = ((diameter / 2))
	if ( (dist + iconDiameter) > mapRadius ) then 
		--Icon is beyond the range of the minimap.
		--Lets set the coords to be on the edge.
		local factor = (mapRadius - iconDiameter) / dist;
		xDist = xDist * factor;
		yDist = yDist * factor;
		return true, xDist, xScale, yDist, yScale
	end
	return false, xDist, xScale, yDist, yScale
end

----------------------------------------------------------------------
function ComputeDistance(fX, fY, tX, tY)							--
-- Return the distance and x/y distance between a set of coords.	--
----------------------------------------------------------------------
	local xDelta = (tX - fX);
	local yDelta = (tY - fY);
	local dist = sqrt(xDelta*xDelta + yDelta*yDelta);
	return dist, xDelta, yDelta
end